/*
 * main.c
 *
 *  Created on: Sep 18, 2010
 *      Author: Thomas Schmid
 */

#include <stdio.h>
#include <string.h>
#include "drivers/cc2520/hton.h"

#include "drivers/mss_watchdog/mss_watchdog.h"
#include "drivers/oled_driver/oled.h"
#include "drivers/mss_uart/mss_uart.h"
#include "drivers/cc2520/cc2520driver.h"
#include "drivers/mss_timer/mss_timer.h"

#define FIRST_CHARACTER 0
#define SRC_ADDRESS 0xaaab

#define DISABLE_TRANSMIT

typedef struct blinkmsg
{
	uint8_t group;
	uint8_t amtype;
	uint16_t counter;
} __attribute__((__packed__)) blinkmsg_t;

typedef enum
{
	OLED,
	RADIO,
} state_t;

uint8_t busy = 0;

state_t main_state = OLED;

char *string1=" LAB 6          ";
char *string2=" Radio Lab      ";

struct oled_data write_data;

__attribute__((__interrupt__)) void Timer2_IRQHandler()
{
	main_state = RADIO;
    MSS_TIM2_clear_irq();
    NVIC_ClearPendingIRQ( Timer2_IRQn );
}


void Radio_sendDone(error_t error)
{
	busy = 0;
	printf("TX done\n\r");
}

void Radio_receive(uint8_t seqn, uint16_t panid, uint16_t saddr, uint16_t daddr, uint8_t* payload, uint8_t length, int8_t rssi)
{
	blinkmsg_t * blinkm = (blinkmsg_t*) payload;

	printf("RX saddr 0x%x seq %d rssi %d count %u\n\r", saddr, seqn, rssi, ntohs(blinkm->counter));
	sprintf(write_data.string1, "d: %5u", daddr);
	sprintf(write_data.string2, "s: %5u, c: %5u", saddr, ntohs(blinkm->counter));
}

int main()
{
    uint8_t seqn = 0;
    blinkmsg_t blinkmsg;
    uint16_t counter = 0;

    blinkmsg.group = 0x3f;
    blinkmsg.amtype = 0x06;
    blinkmsg.counter = 0;

    write_data.line1 = FIRST_LINE;
    write_data.char_offset1 = FIRST_CHARACTER;
    write_data.string1 = string1;

    write_data.line2 = SECOND_LINE;
    write_data.char_offset2 = FIRST_CHARACTER;
    write_data.string2 = string2;

    write_data.contrast_val = 0xFF;

    /* Watchdog Disabling function*/
    MSS_WD_disable();

    /* OLED Initialization */
    OLED_init();


    /* plain Text display */
    OLED_write_data(&write_data,FIRST_TWO_LINES, 1);

    /* Turn on Radio */
    Radio_init();
	RadioState_turnOn();

	/* Start TX timer */
	MSS_TIM2_init(MSS_TIMER_PERIODIC_MODE);
	MSS_TIM2_enable_irq();
	// interrupt every second
	MSS_TIM2_load_immediate(1000 * (g_FrequencyPCLK0/1e3));
	MSS_TIM2_start();

    while(1)
    {
    	switch(main_state)
    	{
    	case OLED:
    		OLED_write_data(&write_data,FIRST_TWO_LINES, 0);
    		break;
    	case RADIO:
    	{
    		if(!busy)
    		{
    			blinkmsg.counter = htons(counter++);
    			printf("TX start\n\r");

    			busy = 1; // we have to do this before the call to send. send done can signal before this function returns!
    			if(Radio_send(seqn++, 0x22, SRC_ADDRESS, 0xffff, (uint8_t*)&blinkmsg, sizeof(blinkmsg_t)) != SUCCESS)
    			{
    				printf("TX failed\n\r");
    				busy = 0;
    			}
    		}
    		main_state = OLED;
    	}
    	}
    }
    return 0;
}
